@@ -44,6 +44,12 @@ typedef enum {
 	UNKNOWN
 } Knowledge;
 
+typedef enum {
+    FILE_META_STATE_INIT = -1,
+    FILE_META_STATE_FALSE = 0,
+    FILE_META_STATE_TRUE = 1,
+} NemoFileMetaState;
+
 struct NemoFileDetails
 {
 	NemoDirectory *directory;
@@ -72,6 +78,7 @@ struct NemoFileDetails
 	time_t atime; /* 0 is unknown */
 	time_t mtime; /* 0 is unknown */
 	time_t ctime; /* 0 is unknown */
+    time_t btime; /* 0 is unknown */
 	
 	char *symlink_name;
 	
@@ -99,9 +106,6 @@ struct NemoFileDetails
     gint thumbnail_throttle_count;
     time_t last_thumbnail_try_mtime;
 
-    GdkPixbuf *scaled_thumbnail;
-    double thumbnail_scale;
-
 	GList *mime_list; /* If this is a directory, the list of MIME types in it. */
 
 	/* Info you might get from a link (.desktop, .directory or nemo link) */
@@ -154,6 +158,10 @@ struct NemoFileDetails
 	eel_boolean_bit get_info_failed               : 1;
 	eel_boolean_bit file_info_is_up_to_date       : 1;
 	
+    eel_boolean_bit got_btime                     : 1;
+    eel_boolean_bit get_btime_failed              : 1;
+    eel_boolean_bit btime_is_up_to_date           : 1;
+
 	eel_boolean_bit got_directory_count           : 1;
 	eel_boolean_bit directory_count_failed        : 1;
 	eel_boolean_bit directory_count_is_up_to_date : 1;
@@ -193,6 +201,8 @@ struct NemoFileDetails
 	eel_boolean_bit is_mountpoint                 : 1;
 	eel_boolean_bit is_hidden                     : 1;
 
+    eel_boolean_bit favorite_checked              : 1;
+
 	eel_boolean_bit has_permissions               : 1;
 	
 	eel_boolean_bit can_read                      : 1;
@@ -213,7 +223,11 @@ struct NemoFileDetails
 
 	eel_boolean_bit filesystem_readonly           : 1;
 	eel_boolean_bit filesystem_use_preview        : 2; /* GFilesystemPreviewType */
-	eel_boolean_bit filesystem_info_is_up_to_date : 1;
+    eel_boolean_bit filesystem_info_is_up_to_date : 1;
+
+    NemoFileLoadDeferredAttrs load_deferred_attrs;
+    NemoFileMetaState pinning;
+    NemoFileMetaState favorite;
 
 	time_t trash_time; /* 0 is unknown */
 
@@ -269,8 +283,6 @@ gboolean      nemo_file_set_display_name               (NemoFile           *file
 							    const char             *display_name,
 							    const char             *edit_name,
 							    gboolean                custom);
-void          nemo_file_set_mount                      (NemoFile           *file,
-							    GMount                 *mount);
 
 /* Mark specified attributes for this file out of date without canceling current
  * I/O or kicking off new I/O.
@@ -297,5 +309,4 @@ void                   nemo_file_operation_complete (NemoFileOperation         *
 							 GFile                         *result_location,
 							 GError                        *error);
 void                   nemo_file_operation_cancel   (NemoFileOperation         *op);
-
 #endif
