@@ -32,6 +32,7 @@
 #include "nemo-file-operations.h"
 #include "nemo-search-directory.h"
 #include "nemo-signaller.h"
+#include "nemo-statx.h"
 #include <eel/eel-glib-extensions.h>
 #include <eel/eel-stock-dialogs.h>
 #include <eel/eel-string.h>
@@ -65,7 +66,7 @@ nemo_compute_title_for_location (GFile *location)
 	if (nemo_is_home_directory (location)) {
 		return g_strdup (_("Home"));
 	}
-	
+
 	builder = NULL;
 	if (location) {
 		file = nemo_file_get (location);
@@ -81,14 +82,19 @@ nemo_compute_title_for_location (GFile *location)
 	}
 
     if (g_settings_get_boolean (nemo_preferences, NEMO_PREFERENCES_SHOW_FULL_PATH_TITLES)) {
+        gchar *uri, *path;
         file = nemo_file_get (location);
-        char *path = g_filename_from_uri (nemo_file_get_uri (file), NULL, NULL);
+
+        uri = nemo_file_get_uri (file);
+        path = g_filename_from_uri (uri, NULL, NULL);
+
         if (path != NULL) {
             title = g_strdup_printf("%s - %s", builder, path);
         } else {
-            title = g_strdup(builder);
+            title = g_strdup_printf("%s - %s", builder, uri);
         }
         nemo_file_unref (file);
+        g_free (uri);
         g_free (path);
         g_free (builder);
     } else {
@@ -98,10 +104,39 @@ nemo_compute_title_for_location (GFile *location)
     return title;
 }
 
+// TODO: Maybe this can replace nemo_compute_title_for_location() all around?
+char *
+nemo_compute_search_title_for_location (GFile *location)
+{
+    GFile *home_file;
+    gchar *location_string;
+
+    if (nemo_is_home_directory (location)) {
+        return g_strdup (_("Home"));
+    }
+
+    home_file = g_file_new_for_path (g_get_home_dir ());
+
+    location_string = NULL;
+
+    location_string = g_file_get_relative_path (home_file, location);
+
+    if (location_string == NULL) {
+        if (g_file_is_native (location)) {
+            location_string = g_file_get_path (location);
+        } else {
+            location_string = g_file_get_uri (location);
+        }
+    }
+
+    g_object_unref (home_file);
+
+    return location_string;
+}
 
 /**
  * nemo_get_user_directory:
- * 
+ *
  * Get the path for the directory containing nemo settings.
  *
  * Return value: the directory path.
@@ -114,13 +149,13 @@ nemo_get_user_directory (void)
 	user_directory = g_build_filename (g_get_user_config_dir (),
 					   NEMO_USER_DIRECTORY_NAME,
 					   NULL);
-	
+
 	if (!g_file_test (user_directory, G_FILE_TEST_EXISTS)) {
 		g_mkdir_with_parents (user_directory, DEFAULT_NEMO_DIRECTORY_MODE);
-		/* FIXME bugzilla.gnome.org 41286: 
-		 * How should we handle the case where this mkdir fails? 
-		 * Note that nemo_application_startup will refuse to launch if this 
-		 * directory doesn't get created, so that case is OK. But the directory 
+		/* FIXME bugzilla.gnome.org 41286:
+		 * How should we handle the case where this mkdir fails?
+		 * Note that nemo_application_startup will refuse to launch if this
+		 * directory doesn't get created, so that case is OK. But the directory
 		 * could be deleted after Nemo was launched, and perhaps
 		 * there is some bad side-effect of not handling that case.
 		 */
@@ -131,7 +166,7 @@ nemo_get_user_directory (void)
 
 /**
  * nemo_get_accel_map_file:
- * 
+ *
  * Get the path for the filename containing nemo accelerator map.
  * The filename need not exist.
  *
@@ -186,7 +221,7 @@ parse_xdg_dirs (const char *config_file)
   gboolean relative;
 
   array = g_array_new (TRUE, TRUE, sizeof (XdgDirEntry));
-  
+
   if (config_file == NULL)
     {
       config_file_free = g_build_filename (g_get_user_config_dir (),
@@ -203,15 +238,15 @@ parse_xdg_dirs (const char *config_file)
 	  p = lines[i];
 	  while (g_ascii_isspace (*p))
 	    p++;
-      
+
 	  if (*p == '#')
 	    continue;
-      
+
 	  value = strchr (p, '=');
 	  if (value == NULL)
 	    continue;
 	  *value++ = 0;
-      
+
 	  g_strchug (g_strchomp (p));
 	  if (!g_str_has_prefix (p, "XDG_"))
 	    continue;
@@ -219,14 +254,14 @@ parse_xdg_dirs (const char *config_file)
 	    continue;
 	  type_start = p + 4;
 	  type_end = p + strlen (p) - 4;
-      
+
 	  while (g_ascii_isspace (*value))
 	    value++;
-      
+
 	  if (*value != '"')
 	    continue;
 	  value++;
-      
+
 	  relative = FALSE;
 	  if (g_str_has_prefix (value, "$HOME"))
 	    {
@@ -237,7 +272,7 @@ parse_xdg_dirs (const char *config_file)
 	    }
 	  else if (*value != '/')
 	    continue;
-	  
+
 	  d = unescaped = g_malloc (strlen (value) + 1);
 	  while (*value && *value != '"')
 	    {
@@ -246,7 +281,7 @@ parse_xdg_dirs (const char *config_file)
 	      *d++ = *value++;
 	    }
 	  *d = 0;
-      
+
 	  *type_end = 0;
 	  dir.type = g_strdup (type_start);
 	  if (relative)
@@ -254,17 +289,17 @@ parse_xdg_dirs (const char *config_file)
 	      dir.path = g_build_filename (g_get_home_dir (), unescaped, NULL);
 	      g_free (unescaped);
 	    }
-	  else 
+	  else
 	    dir.path = unescaped;
-      
+
 	  g_array_append_val (array, dir);
 	}
-      
+
       g_strfreev (lines);
     }
-  
+
   g_free (config_file_free);
-  
+
   return (XdgDirEntry *)g_array_free (array, FALSE);
 }
 
@@ -286,13 +321,13 @@ xdg_dir_changed (NemoFile *file,
 		if (path) {
 			char *argv[5];
 			int i;
-			
+
 			g_free (dir->path);
 			dir->path = path;
 
 			i = 0;
-			argv[i++] = "xdg-user-dirs-update";
-			argv[i++] = "--set";
+			argv[i++] = (char *)"xdg-user-dirs-update";
+			argv[i++] = (char *)"--set";
 			argv[i++] = dir->type;
 			argv[i++] = dir->path;
 			argv[i++] = NULL;
@@ -301,7 +336,7 @@ xdg_dir_changed (NemoFile *file,
 			   if multiple dirs change at the same time. Its
 			   blocking the main thread, but these updates should
 			   be very rare and very fast. */
-			g_spawn_sync (NULL, 
+			g_spawn_sync (NULL,
 				      argv, NULL,
 				      G_SPAWN_SEARCH_PATH |
 				      G_SPAWN_STDOUT_TO_DEV_NULL |
@@ -319,7 +354,7 @@ xdg_dir_changed (NemoFile *file,
 	g_object_unref (dir_location);
 }
 
-static void 
+static void
 xdg_dir_cache_changed_cb (GFileMonitor  *monitor,
 			  GFile *file,
 			  GFile *other_file,
@@ -407,7 +442,7 @@ update_xdg_dir_cache (void)
 	desktop_dir_changed ();
 
 	cached_xdg_dirs = parse_xdg_dirs (NULL);
-	
+
 	for (i = 0 ; cached_xdg_dirs[i].type != NULL; i++) {
 		cached_xdg_dirs[i].file = NULL;
 		if (strcmp (cached_xdg_dirs[i].path, g_get_home_dir ()) != 0) {
@@ -432,7 +467,7 @@ update_xdg_dir_cache (void)
 		g_object_unref (file);
 		g_free (config_file);
 
-		eel_debug_call_at_shutdown (destroy_xdg_dir_cache); 
+		eel_debug_call_at_shutdown (destroy_xdg_dir_cache);
 	}
 }
 
@@ -456,7 +491,7 @@ nemo_get_xdg_dir (const char *type)
 	if (strcmp ("TEMPLATES", type) == 0) {
 		return g_build_filename (g_get_home_dir (), "Templates", NULL);
 	}
-	
+
 	return g_strdup (g_get_home_dir ());
 }
 
@@ -472,7 +507,7 @@ get_desktop_path (void)
 
 /**
  * nemo_get_desktop_directory:
- * 
+ *
  * Get the path for the directory containing files on the desktop.
  *
  * Return value: the directory path.
@@ -481,17 +516,17 @@ char *
 nemo_get_desktop_directory (void)
 {
 	char *desktop_directory;
-	
+
 	desktop_directory = get_desktop_path ();
 
 	/* Don't try to create a home directory */
 	if (!g_settings_get_boolean (nemo_preferences, NEMO_PREFERENCES_DESKTOP_IS_HOME_DIR)) {
 		if (!g_file_test (desktop_directory, G_FILE_TEST_EXISTS)) {
 			g_mkdir (desktop_directory, DEFAULT_DESKTOP_DIRECTORY_MODE);
-			/* FIXME bugzilla.gnome.org 41286: 
-			 * How should we handle the case where this mkdir fails? 
-			 * Note that nemo_application_startup will refuse to launch if this 
-			 * directory doesn't get created, so that case is OK. But the directory 
+			/* FIXME bugzilla.gnome.org 41286:
+			 * How should we handle the case where this mkdir fails?
+			 * Note that nemo_application_startup will refuse to launch if this
+			 * directory doesn't get created, so that case is OK. But the directory
 			 * could be deleted after Nemo was launched, and perhaps
 			 * there is some bad side-effect of not handling that case.
 			 */
@@ -506,7 +541,7 @@ nemo_get_desktop_location (void)
 {
 	char *desktop_directory;
 	GFile *res;
-	
+
 	desktop_directory = get_desktop_path ();
 
 	res = g_file_new_for_path (desktop_directory);
@@ -517,7 +552,7 @@ nemo_get_desktop_location (void)
 
 /**
  * nemo_get_desktop_directory_uri:
- * 
+ *
  * Get the uri for the directory containing files on the desktop.
  *
  * Return value: the directory path.
@@ -527,7 +562,7 @@ nemo_get_desktop_directory_uri (void)
 {
 	char *desktop_path;
 	char *desktop_uri;
-	
+
 	desktop_path = nemo_get_desktop_directory ();
 	desktop_uri = g_filename_to_uri (desktop_path, NULL, NULL);
 	g_free (desktop_path);
@@ -540,7 +575,7 @@ nemo_get_desktop_directory_uri_no_create (void)
 {
 	char *desktop_path;
 	char *desktop_uri;
-	
+
 	desktop_path = get_desktop_path ();
 	desktop_uri = g_filename_to_uri (desktop_path, NULL, NULL);
 	g_free (desktop_path);
@@ -560,7 +595,7 @@ nemo_should_use_templates_directory (void)
 {
 	char *dir;
 	gboolean res;
-	
+
 	dir = nemo_get_xdg_dir ("TEMPLATES");
 	res = strcmp (dir, g_get_home_dir ()) != 0;
 	g_free (dir);
@@ -605,7 +640,7 @@ nemo_get_searches_directory (void)
 	user_dir = nemo_get_user_directory ();
 	searches_dir = g_build_filename (user_dir, "searches", NULL);
 	g_free (user_dir);
-	
+
 	if (!g_file_test (searches_dir, G_FILE_TEST_EXISTS))
 		g_mkdir (searches_dir, DEFAULT_NEMO_DIRECTORY_MODE);
 
@@ -648,7 +683,7 @@ update_desktop_dir (void)
 
 	path = get_desktop_path ();
 	desktop_dir = g_file_new_for_path (path);
-	
+
 	dirname = g_path_get_dirname (path);
 	desktop_dir_dir = g_file_new_for_path (dirname);
 	g_free (dirname);
@@ -663,7 +698,7 @@ nemo_is_home_directory_file (GFile *dir,
 	char *dirname;
 	static GFile *home_dir_dir = NULL;
 	static char *home_dir_filename = NULL;
-	
+
 	if (home_dir_dir == NULL) {
 		dirname = g_path_get_dirname (g_get_home_dir ());
 		home_dir_dir = g_file_new_for_path (dirname);
@@ -679,7 +714,7 @@ gboolean
 nemo_is_home_directory (GFile *dir)
 {
 	static GFile *home_dir = NULL;
-	
+
 	if (home_dir == NULL) {
 		home_dir = g_file_new_for_path (g_get_home_dir ());
 	}
@@ -691,15 +726,15 @@ gboolean
 nemo_is_root_directory (GFile *dir)
 {
 	static GFile *root_dir = NULL;
-	
+
 	if (root_dir == NULL) {
 		root_dir = g_file_new_for_path ("/");
 	}
 
 	return g_file_equal (dir, root_dir);
 }
-		
-		
+
+
 gboolean
 nemo_is_desktop_directory_file (GFile *dir,
 				    const char *file)
@@ -741,7 +776,7 @@ nemo_is_desktop_directory (GFile *dir)
 
 /**
  * nemo_get_gmc_desktop_directory:
- * 
+ *
  * Get the path for the directory containing the legacy gmc desktop.
  *
  * Return value: the directory path.
@@ -766,7 +801,7 @@ nemo_get_data_file_path (const char *partial_path)
 		return path;
 	}
 	g_free (path);
-	
+
 	/* next try the shared directory */
 	path = g_build_filename (NEMO_DATADIR, partial_path, NULL);
 	if (g_file_test (path, G_FILE_TEST_EXISTS)) {
@@ -802,26 +837,26 @@ nemo_ensure_unique_file_name (const char *directory_uri,
 				    extension);
 	child = g_file_get_child (dir, filename);
 	g_free (filename);
-	
+
 	copy = 1;
 	while ((info = g_file_query_info (child, G_FILE_ATTRIBUTE_STANDARD_TYPE, 0, NULL, NULL)) != NULL) {
 		g_object_unref (info);
 		g_object_unref (child);
-		
+
 		filename = g_strdup_printf ("%s-%d%s",
 					    base_name,
 					    copy,
 					    extension);
 		child = g_file_get_child (dir, filename);
 		g_free (filename);
-		
+
 		copy++;
 	}
 
 	res = g_file_get_uri (child);
 	g_object_unref (child);
 	g_object_unref (dir);
-	
+
 	return res;
 }
 
@@ -834,14 +869,14 @@ nemo_unique_temporary_file_name (void)
 
 	file_name = g_strdup_printf ("%sXXXXXX", prefix);
 
-	fd = g_mkstemp (file_name); 
+	fd = g_mkstemp (file_name);
 	if (fd == -1) {
 		g_free (file_name);
 		file_name = NULL;
 	} else {
 		close (fd);
 	}
-	
+
 	return file_name;
 }
 
@@ -866,18 +901,18 @@ nemo_find_existing_uri_in_hierarchy (GFile *location)
 		location = g_file_get_parent (location);
 		g_object_unref (tmp);
 	}
-	
+
 	return location;
 }
 
 /**
  * nemo_find_file_insensitive
- * 
+ *
  * Attempt to find a file case-insentively. If the path can be found, the
  * returned file maps directly to it. Otherwise, a file using the
  * originally-cased path is returned. This function performs might perform
  * I/O.
- * 
+ *
  * Return value: a #GFile to a child specified by @name.
  **/
 GFile *
@@ -887,11 +922,11 @@ nemo_find_file_insensitive (GFile *parent, const gchar *name)
 	gchar *component;
 	GFile *file, *next;
 	gint i;
-	
+
 	split_path = g_strsplit (name, G_DIR_SEPARATOR_S, -1);
-	
+
 	file = g_object_ref (parent);
-	
+
 	for (i = 0; (component = split_path[i]) != NULL; i++) {
 		if (!(next = nemo_find_file_insensitive_next (file,
 		                                                  component))) {
@@ -904,7 +939,7 @@ nemo_find_file_insensitive (GFile *parent, const gchar *name)
 		file = next;
 	}
 	g_strfreev (split_path);
-	
+
 	if (file) {
 		return file;
 	}
@@ -927,10 +962,10 @@ nemo_find_file_insensitive_next (GFile *parent, const gchar *name)
 		return file;
 	}
 	g_object_unref (file);
-	
+
 	ascii_collation_key = g_ascii_strdown (name, -1);
 	use_utf8 = g_utf8_validate (name, -1, NULL);
-	utf8_collation_key = NULL;	
+	utf8_collation_key = NULL;
 	if (use_utf8) {
 		case_folded_name = g_utf8_casefold (name, -1);
 		utf8_collation_key = g_utf8_collate_key (case_folded_name, -1);
@@ -945,10 +980,10 @@ nemo_find_file_insensitive_next (GFile *parent, const gchar *name)
 	if (children != NULL) {
 		while ((info = g_file_enumerator_next_file (children, NULL, NULL))) {
 			child_name = g_file_info_get_name (info);
-			
+
 			if (use_utf8 && g_utf8_validate (child_name, -1, NULL)) {
 				gchar *case_folded;
-				
+
 				case_folded = g_utf8_casefold (child_name, -1);
 				child_key = g_utf8_collate_key (case_folded, -1);
 				g_free (case_folded);
@@ -957,7 +992,7 @@ nemo_find_file_insensitive_next (GFile *parent, const gchar *name)
 				child_key = g_ascii_strdown (child_name, -1);
 				compare_key = ascii_collation_key;
 			}
-			
+
 			found = strcmp (child_key, compare_key) == 0;
 			g_free (child_key);
 			if (found) {
@@ -968,16 +1003,16 @@ nemo_find_file_insensitive_next (GFile *parent, const gchar *name)
 		g_file_enumerator_close (children, NULL, NULL);
 		g_object_unref (children);
 	}
-	
+
 	g_free (ascii_collation_key);
 	g_free (utf8_collation_key);
-	
+
 	if (filename) {
 		file = g_file_get_child (parent, filename);
 		g_free (filename);
 		return file;
 	}
-	
+
 	return NULL;
 }
 
@@ -1014,7 +1049,7 @@ nemo_is_file_roller_installed (void)
 #define GSM_INTERFACE "org.gnome.SessionManager"
 
 /* The following values come from
- * http://www.gnome.org/~mccann/gnome-session/docs/gnome-session.html#org.gnome.SessionManager.Inhibit 
+ * http://www.gnome.org/~mccann/gnome-session/docs/gnome-session.html#org.gnome.SessionManager.Inhibit
  */
 #define INHIBIT_LOGOUT (1U)
 #define INHIBIT_SUSPEND (4U)
@@ -1144,17 +1179,17 @@ nemo_uninhibit_power_manager (gint cookie)
 gboolean
 nemo_is_in_system_dir (GFile *file)
 {
-	const char * const * data_dirs; 
+	const char * const * data_dirs;
 	char *path, *gnome2;
 	int i;
 	gboolean res;
-	
+
 	if (!g_file_is_native (file)) {
 		return FALSE;
 	}
 
 	path = g_file_get_path (file);
-	
+
 	res = FALSE;
 
 	data_dirs = g_get_system_data_dirs ();
@@ -1163,7 +1198,7 @@ nemo_is_in_system_dir (GFile *file)
 			res = TRUE;
 			break;
 		}
-		
+
 	}
 
 	if (!res) {
@@ -1175,7 +1210,7 @@ nemo_is_in_system_dir (GFile *file)
 		g_free (gnome2);
 	}
 	g_free (path);
-	
+
 	return res;
 }
 
@@ -1438,6 +1473,328 @@ nemo_get_cached_x_content_types_for_mount (GMount *mount)
 	return NULL;
 }
 
+static void
+debug_icon_names (const gchar *format, ...)
+{
+    static gboolean debug_removable_device_icons = FALSE;
+    static gsize once_init = 0;
+
+    if (g_once_init_enter (&once_init)) {
+        debug_removable_device_icons = g_getenv ("NEMO_DEBUG_DEVICE_ICONS") != NULL;
+
+        g_once_init_leave (&once_init, 1);
+    }
+
+    if (!debug_removable_device_icons) {
+        return;
+    }
+
+    va_list args;
+    va_start (args, format);
+    g_logv (NULL, G_LOG_LEVEL_MESSAGE, format, args);
+    va_end(args);
+}
+
+static gboolean
+icon_name_is_non_specific (const gchar *icon_name)
+{
+    gint i;
+
+    static const char * non_specific_icon_names[] = { "drive-harddisk-usb-symbolic" };
+
+    for (i = 0; i < G_N_ELEMENTS (non_specific_icon_names); i++) {
+        if (g_strcmp0 (icon_name, non_specific_icon_names[i]) == 0) {
+            return TRUE;
+        }
+    }
+
+    return FALSE;
+}
+
+gchar *
+nemo_get_mount_icon_name (GMount *mount)
+{
+    GDrive *drive;
+    GIcon *gicon;
+    gchar *icon_name;
+    gchar *mount_icon_name;
+    gchar *dev_name;
+
+    g_return_val_if_fail (mount != NULL, NULL);
+
+    dev_name = g_mount_get_name (mount);
+    icon_name = NULL;
+    mount_icon_name = NULL;
+
+    gicon = g_mount_get_symbolic_icon (mount);
+
+    if (G_IS_THEMED_ICON (gicon)) {
+        mount_icon_name = g_strdup (g_themed_icon_get_names (G_THEMED_ICON (gicon))[0]);
+
+        if (icon_name_is_non_specific (mount_icon_name)) {
+            debug_icon_names ("mount %s: icon name '%s' too non-specific", dev_name, mount_icon_name);
+        } else {
+            icon_name = g_strdup (mount_icon_name);
+        }
+    }
+
+    g_clear_object (&gicon);
+
+    if (icon_name != NULL) {
+        debug_icon_names ("mount %s: icon name '%s' is being used", dev_name, icon_name);
+
+        g_free (dev_name);
+        g_free (mount_icon_name);
+
+        return icon_name;
+    }
+
+    drive = g_mount_get_drive (mount);
+
+    if (drive != NULL) {
+        gicon = g_drive_get_symbolic_icon (drive);
+        g_object_unref (drive);
+
+        if (G_IS_THEMED_ICON (gicon)) {
+            icon_name = g_strdup (g_themed_icon_get_names (G_THEMED_ICON (gicon))[0]);
+        }
+
+        g_object_unref (gicon);
+    }
+
+    if (icon_name == NULL) {
+        if (mount_icon_name) {
+            icon_name = g_strdup (mount_icon_name);
+        } else {
+            icon_name = g_strdup ("folder-symbolic"); // any theme will have at least this?...
+        }
+
+        debug_icon_names ("mount %s: no other good icon name found, using fallback of '%s'", dev_name, icon_name);
+    }
+
+    g_free (dev_name);
+    g_free (mount_icon_name);
+
+    return icon_name;
+}
+
+gchar *
+nemo_get_volume_icon_name (GVolume *volume)
+{
+    GDrive *drive;
+    GIcon *gicon;
+    gchar *icon_name;
+    gchar *dev_name;
+    gchar *volume_icon_name;
+
+    g_return_val_if_fail (volume != NULL, NULL);
+
+    dev_name = g_volume_get_name (volume);
+    icon_name = NULL;
+    volume_icon_name = NULL;
+
+    gicon = g_volume_get_symbolic_icon (volume);
+
+    if (G_IS_THEMED_ICON (gicon)) {
+        volume_icon_name = g_strdup (g_themed_icon_get_names (G_THEMED_ICON (gicon))[0]);
+
+        if (icon_name_is_non_specific (volume_icon_name)) {
+            debug_icon_names ("volume %s: icon name '%s' too non-specific", dev_name, volume_icon_name);
+        } else {
+            icon_name = g_strdup (volume_icon_name);
+        }
+    }
+
+    g_clear_object (&gicon);
+
+    if (icon_name != NULL) {
+        debug_icon_names ("volume %s: icon name '%s' is being used", dev_name, icon_name);
+
+        g_free (dev_name);
+        g_free (volume_icon_name);
+
+        return icon_name;
+    }
+
+    drive = g_volume_get_drive (volume);
+
+    if (drive != NULL) {
+        gicon = g_drive_get_symbolic_icon (drive);
+        g_object_unref (drive);
+
+        if (G_IS_THEMED_ICON (gicon)) {
+            icon_name = g_strdup (g_themed_icon_get_names (G_THEMED_ICON (gicon))[0]);
+        }
+
+        g_object_unref (gicon);
+    }
+
+    if (icon_name == NULL) {
+        if (volume_icon_name) {
+            icon_name = g_strdup (volume_icon_name);
+        } else {
+            icon_name = g_strdup ("folder-symbolic"); // any theme will have at least this?...
+
+        }
+
+        debug_icon_names ("volume %s: no good icon name found, using fallback of '%s'", dev_name, icon_name);
+    }
+
+    g_free (dev_name);
+    g_free (volume_icon_name);
+
+    return icon_name;
+}
+
+gchar *
+nemo_get_drive_icon_name (GDrive *drive)
+{
+    GIcon *gicon;
+    gchar *icon_name;
+    gchar *dev_name;
+
+    g_return_val_if_fail (drive != NULL, NULL);
+
+    dev_name = g_drive_get_name (drive);
+
+    gicon = g_drive_get_symbolic_icon (drive);
+
+    if (G_IS_THEMED_ICON (gicon)) {
+        icon_name = g_strdup (g_themed_icon_get_names (G_THEMED_ICON (gicon))[0]);
+    } else {
+        icon_name = g_strdup ("folder-symbolic"); // any theme will have at least this?...
+    }
+
+    g_object_unref (gicon);
+
+    debug_icon_names ("drive %s: returning icon '%s'", dev_name, icon_name);
+
+    g_free (dev_name);
+
+    return icon_name;
+}
+
+gchar *
+nemo_get_best_guess_file_mimetype (const gchar *filename,
+                                   GFileInfo   *info,
+                                   goffset      size)
+{
+    /* This is an attempt to do a better job at identifying file types than
+     * the current gio implementation.
+     *
+     * The current behavior for empty (0 size) size files, is to return
+     * "text/plain" so users can touch a file, or create an empty one in their
+     * file manager, and immediately edit it.
+     *
+     * This behavior currently applies regardless of whether or not a file has
+     * an extension.
+     *
+     * More discussion: https://bugzilla.gnome.org/show_bug.cgi?id=755795
+     *
+     * What we do here instead is take a file's extension into account if the file
+     * is zero-length.  If, by doing so, we have a high confidence that a file is
+     * a certain type, we go ahead and use that type.  We only fall back to Gio's
+     * zero-length implementation if the extension is unknown, or it has none.
+     *
+     * - Files that are NOT zero-length are treated the same as before.
+     * - Files that we are not certain about are treated the same as before.
+     *
+     * Again, this only has an effect on zero-length, known-extension files.  My
+     * argument for this differing from the standing implementation is that if I
+     * create a file with a particular extension, I did so for a reason, and I'm not
+     * going to do something silly like make foo.mp3 and attempt to open it with my
+     * media player.  I do, however, expect that if I touch a foo.h file and open it,
+     * it will open up in my source code editor, and *not* my general purpose plain-
+     * text handler.
+     */
+
+    g_return_val_if_fail (filename != NULL, g_strdup ("application/octet-stream"));
+    g_return_val_if_fail (info != NULL, g_strdup ("application/octet-stream"));
+
+    gchar *mime_type = NULL;
+
+    if (size > 0) {
+        /* Default behavior */
+        mime_type = eel_ref_str_get_unique (g_file_info_get_content_type (info));
+    } else {
+        gboolean uncertain;
+        gchar *guessed_type = NULL;
+
+        /* Only give the file basename, not the full path.  a) We may not have it yet, and
+         * b) we don't want g_content_type_guess to keep going and snoop the file.  This will
+         * keep the guess based entirely on the extension, if there is one.
+         */
+        guessed_type = g_content_type_guess (filename, NULL, 0, &uncertain);
+
+        /* Uncertain means, it's not a registered extension, so we fall back to our (gio's)
+         * normal behavior - text/plain (currently at least.) */
+        if (!uncertain) {
+            mime_type = eel_ref_str_get_unique (guessed_type);
+        } else {
+            mime_type = eel_ref_str_get_unique (g_file_info_get_content_type (info));
+        }
+
+        g_free (guessed_type);
+    }
+
+    return mime_type;
+}
+
+static void
+query_btime_async_thread (GTask         *task,
+                          gpointer       object,
+                          gpointer       task_data,
+                          GCancellable  *cancellable)
+{
+    gchar *path;
+    time_t btime;
+
+    btime = 0;
+
+    path = g_file_get_path (G_FILE (object));
+
+    if (path != NULL) {
+        btime = get_file_btime (path);
+        g_free (path);
+    }
+
+    if (btime > 0) {
+        /* Conveniently, gssize is the same as time_t */
+        g_task_return_int (task, (gssize) btime);
+    } else {
+        g_task_return_error (task,
+                             g_error_new (G_FILE_ERROR,
+                                          G_FILE_ERROR_FAILED,
+                                          "statx failed or not supported"));
+    }
+}
+
+void
+nemo_query_btime_async (GFile               *file,
+                        GCancellable        *cancellable,
+                        GAsyncReadyCallback  callback,
+                        gpointer             user_data)
+{
+  GTask *task;
+
+  task = g_task_new (file, cancellable, callback, user_data);
+  g_task_set_priority (task, G_PRIORITY_DEFAULT);
+  g_task_run_in_thread (task, query_btime_async_thread);
+  g_object_unref (task);
+}
+
+time_t
+nemo_query_btime_finish (GFile         *file,
+                             GAsyncResult  *res,
+                             GError       **error)
+{
+  g_return_val_if_fail (g_task_is_valid (res, file), -1);
+
+  return (time_t) g_task_propagate_int (G_TASK (res), error);
+}
+
+/* End copied section */
+
 #if !defined (NEMO_OMIT_SELF_CHECK)
 
 void
