@@ -31,6 +31,7 @@
 G_BEGIN_DECLS
 
 /* Trash options */
+#define NEMO_PREFERENCES_CONFIRM_MOVE_TO_TRASH	"confirm-move-to-trash"
 #define NEMO_PREFERENCES_CONFIRM_TRASH			"confirm-trash"
 #define NEMO_PREFERENCES_ENABLE_DELETE			"enable-delete"
 #define NEMO_PREFERENCES_SWAP_TRASH_DELETE      "swap-trash-delete"
@@ -92,6 +93,7 @@ typedef enum
 #define NEMO_PREFERENCES_SHOW_LIST_VIEW_ICON_TOOLBAR   "show-list-view-icon-toolbar"
 #define NEMO_PREFERENCES_SHOW_COMPACT_VIEW_ICON_TOOLBAR   "show-compact-view-icon-toolbar"
 #define NEMO_PREFERENCES_SHOW_ROOT_WARNING                "show-root-warning"
+#define NEMO_PREFERENCES_SHOW_SHOW_THUMBNAILS_TOOLBAR     "show-show-thumbnails-toolbar"
 
 /* Which views should be displayed for new windows */
 #define NEMO_WINDOW_STATE_START_WITH_STATUS_BAR		"start-with-status-bar"
@@ -114,6 +116,7 @@ typedef enum
 
 /* The default folder viewer - one of the two enums below */
 #define NEMO_PREFERENCES_DEFAULT_FOLDER_VIEWER		"default-folder-viewer"
+#define NEMO_PREFERENCES_INHERIT_FOLDER_VIEWER		"inherit-folder-viewer"
 
 #define NEMO_PREFERENCES_SHOW_FULL_PATH_TITLES      "show-full-path-titles"
 
@@ -128,8 +131,6 @@ typedef enum
 
 #define NEMO_PREFERENCES_SIDEBAR_BOOKMARK_BREAKPOINT "sidebar-bookmark-breakpoint"
 
-#define NEMO_PREFERENCES_CONTEXT_MENUS_SHOW_ALL_ACTIONS	"context-menus-show-all-actions"
-
 enum
 {
 	NEMO_DEFAULT_FOLDER_VIEWER_ICON_VIEW,
@@ -147,7 +148,6 @@ enum
 
 /* Icon View */
 #define NEMO_PREFERENCES_ICON_VIEW_DEFAULT_ZOOM_LEVEL		"default-zoom-level"
-#define NEMO_PREFERENCES_ICON_VIEW_DEFAULT_USE_TIGHTER_LAYOUT   "default-use-tighter-layout"
 #define NEMO_PREFERENCES_ICON_VIEW_LABELS_BESIDE_ICONS		"labels-beside-icons"
 
 /* Which text attributes appear beneath icon names */
@@ -187,12 +187,13 @@ typedef enum
 {
 	NEMO_SPEED_TRADEOFF_ALWAYS,
 	NEMO_SPEED_TRADEOFF_LOCAL_ONLY,
-	NEMO_SPEED_TRADEOFF_NEVER
+    NEMO_SPEED_TRADEOFF_NEVER
 } NemoSpeedTradeoffValue;
 
 #define NEMO_PREFERENCES_SHOW_DIRECTORY_ITEM_COUNTS "show-directory-item-counts"
 #define NEMO_PREFERENCES_SHOW_IMAGE_FILE_THUMBNAILS	"show-image-thumbnails"
 #define NEMO_PREFERENCES_IMAGE_FILE_THUMBNAIL_LIMIT	"thumbnail-limit"
+#define NEMO_PREFERENCES_INHERIT_SHOW_THUMBNAILS "inherit-show-thumbnails"
 
 typedef enum
 {
@@ -242,6 +243,7 @@ typedef enum
 #define NEMO_PREFERENCES_TOOLTIP_FILE_TYPE             "tooltips-show-file-type"
 #define NEMO_PREFERENCES_TOOLTIP_MOD_DATE              "tooltips-show-mod-date"
 #define NEMO_PREFERENCES_TOOLTIP_ACCESS_DATE           "tooltips-show-access-date"
+#define NEMO_PREFERENCES_TOOLTIP_CREATED_DATE          "tooltips-show-birth-date"
 #define NEMO_PREFERENCES_TOOLTIP_FULL_PATH             "tooltips-show-path"
 
 #define NEMO_PREFERENCES_DISABLE_MENU_WARNING          "disable-menu-warning"
@@ -260,33 +262,43 @@ typedef enum
 #define NEMO_PREFERENCES_CLICK_DOUBLE_PARENT_FOLDER    "click-double-parent-folder"
 
 #define NEMO_PREFERENCES_SAVED_SEARCHES                "saved-searches"
+#define NEMO_PREFERENCES_SHOW_MIME_MAKE_EXECUTABLE     "enable-mime-actions-make-executable"
+#define NEMO_PREFERENCES_DEFERRED_ATTR_PRELOAD_LIMIT   "deferred-attribute-preload-limit"
 
 void nemo_global_preferences_init                      (void);
 void nemo_global_preferences_finalize                  (void);
 char *nemo_global_preferences_get_default_folder_viewer_preference_as_iid (void);
-char *nemo_global_preferences_get_desktop_iid (void);
+gboolean nemo_global_preferences_get_inherit_folder_viewer_preference (void);
+gboolean nemo_global_preferences_get_inherit_show_thumbnails_preference (void);
 gboolean nemo_global_preferences_get_ignore_view_metadata (void);
+int nemo_global_preferences_get_size_prefix_preference (void);
+char *nemo_global_preferences_get_desktop_iid (void);
 gint nemo_global_preferences_get_tooltip_flags (void);
 gboolean nemo_global_preferences_should_load_plugin (const gchar *name, const gchar *key);
+gchar **nemo_global_preferences_get_fileroller_mimetypes (void);
 
-GSettings *nemo_preferences;
-GSettings *nemo_icon_view_preferences;
-GSettings *nemo_list_view_preferences;
-GSettings *nemo_compact_view_preferences;
-GSettings *nemo_desktop_preferences;
-GSettings *nemo_tree_sidebar_preferences;
-GSettings *nemo_window_state;
-GSettings *nemo_plugin_preferences;
-GSettings *gnome_lockdown_preferences;
-GSettings *gnome_background_preferences;
-GSettings *gnome_media_handling_preferences;
-GSettings *gnome_terminal_preferences;
-GSettings *cinnamon_privacy_preferences;
-GSettings *cinnamon_interface_preferences;
+extern GSettings *nemo_preferences;
+extern GSettings *nemo_icon_view_preferences;
+extern GSettings *nemo_list_view_preferences;
+extern GSettings *nemo_compact_view_preferences;
+extern GSettings *nemo_desktop_preferences;
+extern GSettings *nemo_tree_sidebar_preferences;
+extern GSettings *nemo_window_state;
+extern GSettings *nemo_plugin_preferences;
+extern GSettings *nemo_menu_config_preferences;
+extern GSettings *gnome_lockdown_preferences;
+extern GSettings *gnome_background_preferences;
+extern GSettings *gnome_media_handling_preferences;
+extern GSettings *gnome_terminal_preferences;
+extern GSettings *cinnamon_privacy_preferences;
+extern GSettings *cinnamon_interface_preferences;
 
-GTimer    *nemo_startup_timer;
+/* Cached for fast access and used in nemo-file.c for constructing date/time strings */
+extern GTimeZone      *prefs_current_timezone;
+extern gboolean        prefs_current_24h_time_format;
+extern NemoDateFormat  prefs_current_date_format;
 
-gchar    **file_roller_mimetypes;
+extern GTimer    *nemo_startup_timer;
 
 G_END_DECLS
 
