@@ -142,15 +142,18 @@ typedef struct {
 						   NemoIconData *data,
 						   int icon_size,
 						   gboolean for_drag_accept,
-						   gboolean *has_window_open);
+						   gboolean *has_window_open,
+                           gboolean visible);
 	void         (* get_icon_text)            (NemoIconContainer *container,
 						   NemoIconData *data,
 						   char **editable_text,
 						   char **additional_text,
                            gboolean *pinned,
+                           gboolean *fav_unavailable,
 						   gboolean include_invisible);
     void         (* update_icon)              (NemoIconContainer *container,
-                                               NemoIcon          *icon);
+                                               NemoIcon          *icon,
+                                               gboolean           visible);
 	char *       (* get_icon_description)     (NemoIconContainer *container,
 						   NemoIconData *data);
 	int          (* compare_icons)            (NemoIconContainer *container,
@@ -163,6 +166,7 @@ typedef struct {
 						   NemoIconData *data);
     gint         (* get_max_layout_lines_for_pango) (NemoIconContainer *container);
     gint         (* get_max_layout_lines)           (NemoIconContainer *container);
+    gint         (* get_additional_text_line_count) (NemoIconContainer *container);
 
 	/* Queries on icons for subclass/client.
 	 * These must be implemented => These are signals !
@@ -371,4 +375,7 @@ void              nemo_icon_container_widget_to_file_operation_position (NemoIco
 void         nemo_icon_container_setup_tooltip_preference_callback (NemoIconContainer *container);
 void         nemo_icon_container_update_tooltip_text (NemoIconContainer  *container,
                                                       NemoIconCanvasItem *item);
+gint         nemo_icon_container_get_additional_text_line_count (NemoIconContainer *container);
+void         nemo_icon_container_set_ok_to_load_deferred_attrs (NemoIconContainer *container,
+                                                                gboolean           ok);
 #endif /* NEMO_ICON_CONTAINER_H */
