@@ -348,7 +348,8 @@ on_open_folder_clicked (GtkWidget *button, NemoActionConfigWidget *widget)
     nemo_application_open_location (nemo_application_get_singleton (),
                                     location,
                                     NULL,
-                                    "nemo");
+                                    "nemo",
+                                    FALSE);
 
     g_free (path);
     g_object_unref (location);
