@@ -50,7 +50,7 @@ struct _NemoConnectServerDialogDetails {
 
 	GtkWidget *info_bar;
 	GtkWidget *info_bar_content;
-	
+
 	GtkWidget *type_combo;
 	GtkWidget *server_entry;
 	GtkWidget *share_entry;
@@ -87,7 +87,7 @@ static void iconized_entry_changed_cb (GtkEditable *entry,
 
 enum {
 	RESPONSE_CONNECT
-};	
+};
 
 struct MethodInfo {
 	const char *scheme;
@@ -97,15 +97,12 @@ struct MethodInfo {
 
 /* A collection of flags for MethodInfo.flags */
 enum {
-	DEFAULT_METHOD = (1 << 0),
-	
-	/* Widgets to display in connect_dialog_setup_for_type */
-	SHOW_SHARE     = (1 << 1),
-	SHOW_PORT      = (1 << 2),
-	SHOW_USER      = (1 << 3),
-	SHOW_DOMAIN    = (1 << 4),
-	
-	IS_ANONYMOUS   = (1 << 5)
+    /* Widgets to display in connect_dialog_setup_for_type */
+    SHOW_SHARE     = (1 << 0),
+    SHOW_PORT      = (1 << 1),
+    SHOW_USER      = (1 << 2),
+    SHOW_DOMAIN    = (1 << 3),
+    IS_ANONYMOUS   = (1 << 4)
 };
 
 /* Remember to fill in descriptions below */
@@ -113,7 +110,7 @@ static struct MethodInfo methods[] = {
 	/* FIXME: we need to alias ssh to sftp */
 	{ "sftp",  SHOW_PORT | SHOW_USER, 22 },
 	{ "ftp",  SHOW_PORT | SHOW_USER, 21 },
-	{ "ftp",  DEFAULT_METHOD | IS_ANONYMOUS | SHOW_PORT, 21 },
+	{ "ftp",  IS_ANONYMOUS | SHOW_PORT, 21 },
 	{ "smb",  SHOW_SHARE | SHOW_USER | SHOW_DOMAIN, 0 },
 	{ "dav",  SHOW_PORT | SHOW_USER, 80 },
 	/* FIXME: hrm, shouldn't it work? */
@@ -167,7 +164,7 @@ connect_dialog_set_connecting (NemoConnectServerDialog *dialog)
 	gint width, height;
 
 	connect_dialog_restore_info_bar (dialog, GTK_MESSAGE_INFO);
-	gtk_widget_show (dialog->details->info_bar);	
+	gtk_widget_show (dialog->details->info_bar);
 
 	content_area = gtk_info_bar_get_content_area (GTK_INFO_BAR (dialog->details->info_bar));
 
@@ -207,7 +204,7 @@ connect_dialog_gvfs_error (NemoConnectServerDialog *dialog)
 	image = gtk_image_new_from_stock (GTK_STOCK_DIALOG_ERROR, GTK_ICON_SIZE_SMALL_TOOLBAR);
 	gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 6);
 	gtk_widget_show (image);
-	
+
 	label = gtk_label_new (_("Can't load the supported server method list.\n"
 				 "Please check your gvfs installation."));
 	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 6);
@@ -235,7 +232,7 @@ iconized_entry_restore (gpointer data,
 
 	g_signal_handlers_disconnect_by_func (entry,
 					      iconized_entry_changed_cb,
-					      dialog);	
+					      dialog);
 }
 
 static void
@@ -302,7 +299,7 @@ connect_dialog_set_info_bar_error (NemoConnectServerDialog *dialog,
 
 		g_free (str);
 
-		break;		
+		break;
 	case G_IO_ERROR_FAILED:
 	default:
 		label = gtk_label_new (error->message);
@@ -518,7 +515,7 @@ connect_dialog_connect_to_server (NemoConnectServerDialog *dialog)
 	int index;
 	GtkTreeIter iter;
 	char *user, *initial_path, *server, *folder, *domain, *port_str;
-	char *t, *join, *uri;
+	char *t, *uri;
 	char *temp, *stripped_server;
 	double port;
 
@@ -526,7 +523,7 @@ connect_dialog_connect_to_server (NemoConnectServerDialog *dialog)
 	gtk_combo_box_get_active_iter (GTK_COMBO_BOX (dialog->details->type_combo), &iter);
 	gtk_tree_model_get (gtk_combo_box_get_model (GTK_COMBO_BOX (dialog->details->type_combo)),
 			    &iter, 0, &index, -1);
-	g_assert (index < G_N_ELEMENTS (methods) && index >= 0);
+	g_assert (index < (int)G_N_ELEMENTS (methods) && index >= 0);
 	meth = &(methods[index]);
 
 	server = gtk_editable_get_chars (GTK_EDITABLE (dialog->details->server_entry), 0, -1);
@@ -547,7 +544,7 @@ connect_dialog_connect_to_server (NemoConnectServerDialog *dialog)
 	/* FTP special case */
 	if (meth->flags & IS_ANONYMOUS) {
 		user = g_strdup ("anonymous");
-		
+
 		/* SMB special case */
 	} else if (strcmp (meth->scheme, "smb") == 0) {
 		g_free (initial_path);
@@ -567,8 +564,8 @@ connect_dialog_connect_to_server (NemoConnectServerDialog *dialog)
 
 	/* domain */
 	domain = gtk_editable_get_chars (GTK_EDITABLE (dialog->details->domain_entry), 0, -1);
-			
-	if (strlen (domain) != 0) {
+
+	if (strlen (domain) != 0 && strcmp(meth->scheme, "smb") == 0) {
 		t = user;
 
 		user = g_strconcat (domain , ";" , t, NULL);
@@ -578,15 +575,12 @@ connect_dialog_connect_to_server (NemoConnectServerDialog *dialog)
 	/* folder */
 	folder = gtk_editable_get_chars (GTK_EDITABLE (dialog->details->folder_entry), 0, -1);
 
-	if (folder[0] != 0 &&
-	    folder[0] != '/') {
-		join = "/";
-	} else {
-		join = "";
-	}
-
 	t = folder;
-	folder = g_strconcat (initial_path, join, t, NULL);
+        if (folder[0] != 0 && folder[0] != '/') {
+	    folder = g_strconcat (initial_path, "/", t, NULL);
+        } else {
+	    folder = g_strconcat (initial_path, "", t, NULL);
+        }
 	g_free (t);
 
 	t = folder;
@@ -606,7 +600,7 @@ connect_dialog_connect_to_server (NemoConnectServerDialog *dialog)
 	uri = g_strdup_printf ("%s://%s%s%s%s%s%s",
 			       meth->scheme,
 			       (user != NULL) ? user : "",
-			       (user[0] != 0) ? "@" : "",
+			       (user != NULL && user[0] != 0) ? "@" : "",
 			       server,
 			       (port_str != NULL) ? ":" : "",
 			       (port_str != NULL) ? port_str : "",
@@ -681,8 +675,6 @@ connect_dialog_response_cb (NemoConnectServerDialog *dialog,
 			    int response_id,
 			    gpointer data)
 {
-	GError *error;
-
 	switch (response_id) {
 	case RESPONSE_CONNECT:
 		connect_to_server_or_finish_fill (dialog);
@@ -692,17 +684,6 @@ connect_dialog_response_cb (NemoConnectServerDialog *dialog,
 	case GTK_RESPONSE_CANCEL:
 		connect_dialog_destroy (dialog);
 		break;
-	case GTK_RESPONSE_HELP :
-		error = NULL;
-		gtk_show_uri (gtk_window_get_screen (GTK_WINDOW (dialog)),
-			      "help:gnome-help/nemo-connect",
-			      gtk_get_current_event_time (), &error);
-		if (error) {
-			eel_show_error_dialog (_("There was an error displaying help."), error->message,
-					       GTK_WINDOW (dialog));
-			g_error_free (error);
-		}
-		break;
 	default :
 		g_assert_not_reached ();
 	}
@@ -763,9 +744,13 @@ connect_dialog_setup_for_type (NemoConnectServerDialog *dialog)
 
 	gtk_tree_model_get (gtk_combo_box_get_model (GTK_COMBO_BOX (dialog->details->type_combo)),
 			    &iter, 0, &index, -1);
-	g_assert (index < G_N_ELEMENTS (methods) && index >= 0);
+	g_assert (index < (int)G_N_ELEMENTS (methods) && index >= 0);
 	meth = &(methods[index]);
 
+    if (g_settings_get_int (nemo_preferences, NEMO_PREFERENCES_LAST_SERVER_CONNECT_METHOD) != index) {
+        g_settings_set_int (nemo_preferences, NEMO_PREFERENCES_LAST_SERVER_CONNECT_METHOD, index);
+    }
+
 	g_object_set (dialog->details->share_entry,
 		      "visible",
 		      (meth->flags & SHOW_SHARE) != 0,
@@ -833,8 +818,8 @@ nemo_connect_server_dialog_init (NemoConnectServerDialog *dialog)
 	GtkListStore *store;
 	GtkCellRenderer *renderer;
 	gchar *str;
-	int i;
-	
+	guint i;
+
 	dialog->details = G_TYPE_INSTANCE_GET_PRIVATE (dialog, NEMO_TYPE_CONNECT_SERVER_DIALOG,
 						       NemoConnectServerDialogDetails);
 
@@ -861,6 +846,7 @@ nemo_connect_server_dialog_init (NemoConnectServerDialog *dialog)
 	label = gtk_label_new (NULL);
 	str = g_strdup_printf ("<b>%s</b>", _("Server Details"));
 	gtk_label_set_markup (GTK_LABEL (label), str);
+	g_free (str);
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 	gtk_box_pack_start (GTK_BOX (content_area), label, FALSE, FALSE, 6);
 	gtk_size_group_add_widget (dialog->details->labels_size_group, label);
@@ -931,6 +917,8 @@ nemo_connect_server_dialog_init (NemoConnectServerDialog *dialog)
 	gtk_size_group_add_widget (dialog->details->contents_size_group, dialog->details->type_combo);
 	gtk_label_set_mnemonic_widget (GTK_LABEL (label), dialog->details->type_combo);
 
+    gint last = g_settings_get_int (nemo_preferences, NEMO_PREFERENCES_LAST_SERVER_CONNECT_METHOD);
+
 	/* each row contains: method index, textual description */
 	store = gtk_list_store_new (2, G_TYPE_INT, G_TYPE_STRING);
 	gtk_combo_box_set_model (GTK_COMBO_BOX (combo), GTK_TREE_MODEL (store));
@@ -943,7 +931,7 @@ nemo_connect_server_dialog_init (NemoConnectServerDialog *dialog)
 	for (i = 0; i < G_N_ELEMENTS (methods); i++) {
 		GtkTreeIter iter;
 		const gchar * const *supported;
-		int j;
+		guint j;
 
 		/* skip methods that don't have corresponding gvfs uri schemes */
 		supported = g_vfs_get_supported_uri_schemes (g_vfs_get_default ());
@@ -966,14 +954,13 @@ nemo_connect_server_dialog_init (NemoConnectServerDialog *dialog)
 
 		gtk_list_store_append (store, &iter);
 		gtk_list_store_set (store, &iter,
-				    0, i,
+				    0, (int)i,
 				    1, get_method_description (&(methods[i])),
 				    -1);
 
-
-		if (methods[i].flags & DEFAULT_METHOD) {
-			gtk_combo_box_set_active_iter (GTK_COMBO_BOX (combo), &iter);
-		}
+        if ((int)i == last) {
+            gtk_combo_box_set_active_iter (GTK_COMBO_BOX (combo), &iter);
+        }
 	}
 
 	if (gtk_combo_box_get_active (GTK_COMBO_BOX (combo)) < 0) {
@@ -1025,6 +1012,7 @@ nemo_connect_server_dialog_init (NemoConnectServerDialog *dialog)
 	label = gtk_label_new (NULL);
 	str = g_strdup_printf ("<b>%s</b>", _("User Details"));
 	gtk_label_set_markup (GTK_LABEL (label), str);
+	g_free (str);
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 	gtk_size_group_add_widget (dialog->details->labels_size_group, label);
 	gtk_box_pack_start (GTK_BOX (content_area), label, FALSE, FALSE, 6);
@@ -1099,9 +1087,6 @@ nemo_connect_server_dialog_init (NemoConnectServerDialog *dialog)
 
 	bind_visibility (dialog, dialog->details->password_entry, checkbox);
 
-        gtk_dialog_add_button (GTK_DIALOG (dialog),
-                               GTK_STOCK_HELP,
-                               GTK_RESPONSE_HELP);
 	gtk_dialog_add_button (GTK_DIALOG (dialog),
 			       GTK_STOCK_CANCEL,
 			       GTK_RESPONSE_CANCEL);
@@ -1214,7 +1199,7 @@ nemo_connect_server_dialog_fill_details_async (NemoConnectServerDialog *self,
 			g_mount_operation_set_password (G_MOUNT_OPERATION (operation),
 							str);
 			set_flags ^= G_ASK_PASSWORD_NEED_PASSWORD;
-			
+
 			if (flags & G_ASK_PASSWORD_SAVING_SUPPORTED &&
 			    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->details->remember_checkbox))) {
 				g_mount_operation_set_password_save (G_MOUNT_OPERATION (operation),
