@@ -100,6 +100,7 @@ struct FileEntry {
 	GSequenceIter *ptr;
 	guint loaded : 1;
     guint expanding : 1;
+    guint ok_to_show_thumb : 1;
 };
 
 G_DEFINE_TYPE_WITH_CODE (NemoListModel, nemo_list_model, G_TYPE_OBJECT,
@@ -314,9 +315,13 @@ nemo_list_model_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter, int colu
 			icon_size = nemo_get_list_icon_size_for_zoom_level (zoom_level);
             icon_scale = nemo_list_model_get_icon_scale (model);
 
-			flags = NEMO_FILE_ICON_FLAGS_USE_THUMBNAILS |
-				NEMO_FILE_ICON_FLAGS_FORCE_THUMBNAIL_SIZE |
+			flags = NEMO_FILE_ICON_FLAGS_FORCE_THUMBNAIL_SIZE |
 				NEMO_FILE_ICON_FLAGS_USE_MOUNT_ICON_AS_EMBLEM;
+
+            if (file_entry->ok_to_show_thumb) {
+                flags |= NEMO_FILE_ICON_FLAGS_USE_THUMBNAILS;
+            }
+
 			if (model->details->drag_view != NULL) {
 				GtkTreePath *path_a, *path_b;
 
@@ -409,11 +414,24 @@ nemo_list_model_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter, int colu
         g_value_init (value, G_TYPE_INT);
 
         if (file != NULL) {
-            g_value_set_int (value, nemo_file_get_pinning (file) ? PINNED_TEXT_WEIGHT : UNPINNED_TEXT_WEIGHT);
-        } else {
-            g_value_set_int (value, UNPINNED_TEXT_WEIGHT);
+            if (nemo_file_is_unavailable_favorite (file)) {
+                g_value_set_int (value, UNAVAILABLE_TEXT_WEIGHT);
+            }
+            else
+            if (nemo_file_get_pinning (file) && !nemo_file_is_in_favorites (file)) {
+                g_value_set_int (value, PINNED_TEXT_WEIGHT);
+            }
+            else
+            {
+                g_value_set_int (value, NORMAL_TEXT_WEIGHT);
+            }
         }
+        break;
+    case NEMO_LIST_MODEL_ICON_SHOWN:
+        g_value_init (value, G_TYPE_BOOLEAN);
 
+        g_value_set_boolean (value, file_entry->ok_to_show_thumb);
+        file_entry->ok_to_show_thumb = TRUE;
         break;
  	default:
  		if (column >= NEMO_LIST_MODEL_NUM_COLUMNS || column < (int)(NEMO_LIST_MODEL_NUM_COLUMNS + model->details->columns->len)) {
@@ -992,7 +1010,8 @@ nemo_list_model_add_file (NemoListModel *model, NemoFile *file,
 	file_entry->parent = NULL;
 	file_entry->subdirectory = NULL;
 	file_entry->files = NULL;
-
+    file_entry->ok_to_show_thumb =
+        nemo_file_get_load_deferred_attrs (file) == NEMO_FILE_LOAD_DEFERRED_ATTRS_PRELOAD;
 	files = model->details->files;
 	parent_hash = model->details->top_reverse_map;
 
@@ -1728,6 +1747,8 @@ nemo_list_model_subdirectory_done_loading (NemoListModel *model, NemoDirectory *
 		return;
 	}
 
+
+
 	file_entry = g_sequence_get (parent_ptr);
 	files = file_entry->files;
 
@@ -1829,4 +1850,4 @@ nemo_list_model_set_expanding (NemoListModel *model, NemoDirectory *directory)
 
     entry = g_sequence_get (ptr);
     entry->expanding = TRUE;
-}
\ No newline at end of file
+}
