@@ -195,10 +195,11 @@ nemo_main_application_create_window (NemoApplication *application,
 		gtk_window_unmaximize (GTK_WINDOW (window));
 	}
 
-	geometry_string = g_settings_get_string
-		(nemo_window_state, NEMO_WINDOW_STATE_GEOMETRY);
-	if (geometry_string != NULL &&
-	    geometry_string[0] != 0) {
+    geometry_string = g_settings_get_string (nemo_window_state, NEMO_WINDOW_STATE_GEOMETRY);
+
+    if (NEMO_MAIN_APPLICATION (application)->priv->geometry == NULL && 
+        geometry_string != NULL &&
+        geometry_string[0] != 0) {
 		/* Ignore saved window position if a window with the same
 		 * location is already showing. That way the two windows
 		 * wont appear at the exact same location on the screen.
@@ -210,6 +211,7 @@ nemo_main_application_create_window (NemoApplication *application,
 			 NEMO_WINDOW_MIN_HEIGHT,
 			 TRUE);
 	}
+
 	g_free (geometry_string);
 
     nemo_undo_manager_attach (application->undo_manager, G_OBJECT (window));
@@ -324,7 +326,7 @@ open_window (NemoMainApplication *application,
 	uri = g_file_get_uri (location);
 	DEBUG ("Opening new window at uri %s", uri);
 
-	window = nemo_main_application_create_window (application,
+	window = nemo_main_application_create_window (NEMO_APPLICATION (application),
 						     screen);
 	nemo_window_go_to (window, location);
 
@@ -345,12 +347,57 @@ open_window (NemoMainApplication *application,
 	g_free (uri);
 }
 
+static void
+open_tabs (NemoMainApplication *application,
+         GFile **locations,
+         guint n_files,
+         GdkScreen *screen,
+         const char *geometry)
+{
+    NemoWindow *window;
+    gchar *uri;
+    gboolean have_geometry;
+
+    window = nemo_main_application_create_window (NEMO_APPLICATION (application),
+                             screen);
+
+    /* open all locations */
+    uri = g_file_get_uri (locations[0]);
+    g_debug ("Opening new tab at uri %s\n", uri);
+    nemo_window_go_to (window, locations[0]);
+    g_free (uri);
+    for (int i = 1; i < n_files; i++) {
+        /* open tabs in reverse order because each
+         * tab is opened before the previous one */
+        guint tab = n_files-i;
+        uri = g_file_get_uri (locations[tab]);
+        g_debug ("Opening new tab at uri %s\n", uri);
+        nemo_window_go_to_tab (window, locations[tab]);
+        g_free (uri);
+    }
+
+    have_geometry = geometry != NULL && strcmp(geometry, "") != 0;
+
+    if (have_geometry && !gtk_widget_get_visible (GTK_WIDGET (window))) {
+        /* never maximize windows opened from shell if a
+         * custom geometry has been requested.
+         */
+        gtk_window_unmaximize (GTK_WINDOW (window));
+        eel_gtk_window_set_initial_geometry_from_string (GTK_WINDOW (window),
+                                 geometry,
+                                 APPLICATION_WINDOW_MIN_WIDTH,
+                                 APPLICATION_WINDOW_MIN_HEIGHT,
+                                 FALSE);
+    }
+}
+
 static void
 open_windows (NemoMainApplication *application,
 	      GFile **files,
 	      gint n_files,
 	      GdkScreen *screen,
-	      const char *geometry)
+	      const char *geometry,
+	      gboolean open_in_tabs)
 {
 	gint i;
 
@@ -358,9 +405,14 @@ open_windows (NemoMainApplication *application,
 		/* Open a window pointing at the default location. */
 		open_window (application, NULL, screen, geometry);
 	} else {
-		/* Open windows at each requested location. */
-		for (i = 0; i < n_files; i++) {
-			open_window (application, files[i], screen, geometry);
+		if (open_in_tabs) {
+			/* Open one window with one tab at each requested location */
+			open_tabs (application, files, n_files, screen, geometry);
+		} else {
+			/* Open windows at each requested location. */
+			for (i = 0; i < n_files; i++) {
+				open_window (application, files[i], screen, geometry);
+			}
 		}
 	}
 }
@@ -369,7 +421,8 @@ static void
 nemo_main_application_open_location (NemoApplication     *application,
                                      GFile               *location,
                                      GFile               *selection,
-                                     const char          *startup_id)
+                                     const char          *startup_id,
+                                     const gboolean      open_in_tabs)
 {
 	NemoWindow *window;
 	GList *sel_list = NULL;
@@ -381,8 +434,13 @@ nemo_main_application_open_location (NemoApplication     *application,
 		sel_list = g_list_prepend (sel_list, nemo_file_get (selection));
 	}
 
-	nemo_window_slot_open_location_full (nemo_window_get_active_slot (window), location,
+	if(open_in_tabs){
+		nemo_window_slot_open_location_full (nemo_window_get_active_slot (window), location,
+						 NEMO_WINDOW_OPEN_FLAG_NEW_TAB, sel_list, NULL, NULL);
+	} else {
+		nemo_window_slot_open_location_full (nemo_window_get_active_slot (window), location,
 						 0, sel_list, NULL, NULL);
+	}
 
 	if (sel_list != NULL) {
 		nemo_file_list_free (sel_list);
@@ -393,13 +451,34 @@ static void
 nemo_main_application_open (GApplication *app,
                             GFile       **files,
                             gint          n_files,
-                            const gchar  *geometry)
+                            const gchar  *options)
 {
 	NemoMainApplication *self = NEMO_MAIN_APPLICATION (app);
 
-	DEBUG ("Open called on the GApplication instance; %d files", n_files);
+	gboolean open_in_tabs = FALSE;
+	gchar *geometry = NULL;
+	const char splitter = '=';
 
-	open_windows (self, files, n_files, gdk_screen_get_default (), geometry);
+	g_debug ("Open called on the GApplication instance; %d files", n_files);
+
+	/* Check if local command line passed --geometry or --tabs */
+	if (strlen (options) > 0) {
+		gchar** split_options = g_strsplit (options, &splitter, 2);
+		if (strcmp (split_options[0], "NULL") != 0) {
+			geometry = g_strdup (split_options[0]);
+		}
+		sscanf (split_options[1], "%d", &open_in_tabs);
+		g_strfreev (split_options);
+	}
+
+	DEBUG ("Open called on the GApplication instance; %d files, open in tabs: %s, geometry: '%s'",
+           n_files,
+           open_in_tabs ? "yes" : "no",
+           geometry ? geometry : "none");
+
+	open_windows (self, files, n_files, gdk_screen_get_default (), geometry, open_in_tabs);
+
+    g_clear_pointer (&geometry, g_free);
 }
 
 static void
@@ -433,6 +512,7 @@ do_cmdline_sanity_checks (NemoMainApplication *self,
 			  gboolean perform_self_check,
 			  gboolean version,
 			  gboolean kill_shell,
+			  gboolean open_in_tabs,
 			  gchar **remaining)
 {
 	gboolean retval = FALSE;
@@ -450,6 +530,7 @@ do_cmdline_sanity_checks (NemoMainApplication *self,
 	}
 
 	if (self->priv->geometry != NULL &&
+	    !open_in_tabs &&
 	    remaining != NULL && remaining[0] != NULL && remaining[1] != NULL) {
 		g_printerr ("%s\n",
 			    _("--geometry cannot be used with more than one URI."));
@@ -488,10 +569,12 @@ nemo_main_application_local_command_line (GApplication *application,
 	gboolean perform_self_check = FALSE;
 	gboolean version = FALSE;
 	gboolean browser = FALSE;
+	gboolean open_in_tabs = FALSE;
 	gboolean kill_shell = FALSE;
 	gboolean no_default_window = FALSE;
     gboolean no_desktop_ignored = FALSE;
 	gboolean fix_cache = FALSE;
+    gboolean debug = FALSE;
 	gchar **remaining = NULL;
     GApplicationFlags init_flags;
 	NemoMainApplication *self = NEMO_MAIN_APPLICATION (application);
@@ -507,13 +590,18 @@ nemo_main_application_local_command_line (GApplication *application,
 		{ "version", '\0', 0, G_OPTION_ARG_NONE, &version,
 		  N_("Show the version of the program."), NULL },
 		{ "geometry", 'g', 0, G_OPTION_ARG_STRING, &self->priv->geometry,
-		  N_("Create the initial window with the given geometry."), N_("GEOMETRY") },
+		  N_("Create the initial window with the given geometry. "
+             "Examples: nemo --geometry=+100+100, nemo --geometry=600x400, nemo --geometry=600x400+100+100."), N_("GEOMETRY") },
 		{ "no-default-window", 'n', 0, G_OPTION_ARG_NONE, &no_default_window,
 		  N_("Only create windows for explicitly specified URIs."), NULL },
         { "no-desktop", '\0', 0, G_OPTION_ARG_NONE, &no_desktop_ignored,
           N_("Ignored argument - left for compatibility only."), NULL },
+		{ "tabs", 't', 0, G_OPTION_ARG_NONE, &open_in_tabs,
+		  N_("Open URIs in tabs."), NULL },
 		{ "fix-cache", '\0', 0, G_OPTION_ARG_NONE, &fix_cache,
 		  N_("Repair the user thumbnail cache - this can be useful if you're having trouble with file thumbnails.  Must be run as root"), NULL },
+        { "debug", 0, 0, G_OPTION_ARG_NONE, &debug,
+          "Enable debugging code.  Example usage: 'NEMO_DEBUG=Actions,Window nemo --debug'.  Use NEMO_DEBUG=help for more topics.", NULL },
 		{ "quit", 'q', 0, G_OPTION_ARG_NONE, &kill_shell, 
 		  N_("Quit Nemo."), NULL },
 		{ G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_STRING_ARRAY, &remaining, NULL,  N_("[URI...]") },
@@ -547,8 +635,12 @@ nemo_main_application_local_command_line (GApplication *application,
 		goto out;
 	}
 
+    if (debug) {
+        g_setenv ("G_MESSAGES_DEBUG", "all", TRUE);
+    }
+
 	if (!do_cmdline_sanity_checks (self, perform_self_check,
-				       version, kill_shell, remaining)) {
+				       version, kill_shell, open_in_tabs, remaining)) {
 		*exit_status = EXIT_FAILURE;
 		goto out;
 	}
@@ -643,11 +735,14 @@ post_registration:
 	}
 	/* Invoke "Open" to create new windows */
 	if (len > 0) {
-		if (self->priv->geometry != NULL) {
-			g_application_open (application, files, len, self->priv->geometry);
+		gchar* concatOptions = g_malloc0(64);
+		if (self->priv->geometry == NULL) {
+			g_snprintf (concatOptions, 64, "NULL=%d", open_in_tabs);
 		} else {
-			g_application_open (application, files, len, "");
+			g_snprintf (concatOptions, 64, "%s=%d", self->priv->geometry, open_in_tabs);
 		}
+		g_application_open (application, files, len, concatOptions);
+		g_free (concatOptions);
 	}
 
 	for (idx = 0; idx < len; idx++) {
