@@ -54,6 +54,7 @@ struct _NemoToolbarPriv {
     GtkWidget *icon_view_button;
     GtkWidget *list_view_button;
     GtkWidget *compact_view_button;
+    GtkWidget *show_thumbnails_button;
 
     GtkToolItem *navigation_box;
     GtkToolItem *refresh_box;
@@ -85,7 +86,7 @@ G_DEFINE_TYPE (NemoToolbar, nemo_toolbar, GTK_TYPE_BOX);
 static void
 nemo_toolbar_update_root_state (NemoToolbar *self)
 {
-    if (geteuid() == 0) {
+    if (geteuid() == 0 && g_settings_get_boolean (nemo_preferences, NEMO_PREFERENCES_SHOW_ROOT_WARNING)) {
         if (self->priv->show_root_bar != TRUE) {
             self->priv->show_root_bar = TRUE;
         }
@@ -183,6 +184,11 @@ toolbar_update_appearance (NemoToolbar *self)
     icon_toolbar = g_settings_get_boolean (nemo_preferences, NEMO_PREFERENCES_SHOW_COMPACT_VIEW_ICON_TOOLBAR);
     if ( icon_toolbar == FALSE ) { gtk_widget_hide (widgetitem); }
     else {gtk_widget_show (GTK_WIDGET(widgetitem));}
+    
+    widgetitem = self->priv->show_thumbnails_button;
+    icon_toolbar = g_settings_get_boolean (nemo_preferences, NEMO_PREFERENCES_SHOW_SHOW_THUMBNAILS_TOOLBAR);
+    if ( icon_toolbar == FALSE ) { gtk_widget_hide (widgetitem); }
+    else {gtk_widget_show (GTK_WIDGET(widgetitem));}
 
     if (gtk_widget_get_visible(self->priv->previous_button) == FALSE &&
         gtk_widget_get_visible(self->priv->next_button) == FALSE &&
@@ -211,7 +217,8 @@ toolbar_update_appearance (NemoToolbar *self)
     if (gtk_widget_get_visible(self->priv->search_button) == FALSE &&
         gtk_widget_get_visible(self->priv->new_folder_button) == FALSE && 
         gtk_widget_get_visible(self->priv->open_terminal_button) == FALSE &&
-        gtk_widget_get_visible(self->priv->toggle_location_button) == FALSE)
+        gtk_widget_get_visible(self->priv->toggle_location_button) == FALSE &&
+        gtk_widget_get_visible(self->priv->show_thumbnails_button) == FALSE)
     {
         gtk_widget_hide(GTK_WIDGET (self->priv->tools_box));
     } else {
@@ -391,6 +398,9 @@ nemo_toolbar_constructed (GObject *obj)
 
     self->priv->search_button = toolbar_create_toolbutton (self, TRUE, NEMO_ACTION_SEARCH);
     gtk_container_add (GTK_CONTAINER (box), self->priv->search_button);
+    
+    self->priv->show_thumbnails_button = toolbar_create_toolbutton (self, TRUE, NEMO_ACTION_SHOW_THUMBNAILS);
+    gtk_container_add (GTK_CONTAINER (box), self->priv->show_thumbnails_button);
 
     gtk_style_context_add_class (gtk_widget_get_style_context (box), GTK_STYLE_CLASS_RAISED);
     gtk_style_context_add_class (gtk_widget_get_style_context (box), GTK_STYLE_CLASS_LINKED);
@@ -465,6 +475,12 @@ nemo_toolbar_constructed (GObject *obj)
     g_signal_connect_swapped (nemo_preferences,
                   "changed::" NEMO_PREFERENCES_SHOW_COMPACT_VIEW_ICON_TOOLBAR,
                   G_CALLBACK (toolbar_update_appearance), self);
+    g_signal_connect_swapped (nemo_preferences,
+                  "changed::" NEMO_PREFERENCES_SHOW_SHOW_THUMBNAILS_TOOLBAR,
+                  G_CALLBACK (toolbar_update_appearance), self);
+    g_signal_connect_swapped (nemo_preferences,
+                  "changed::" NEMO_PREFERENCES_SHOW_IMAGE_FILE_THUMBNAILS,
+                  G_CALLBACK (toolbar_update_appearance), self);
 
 	toolbar_update_appearance (self);
 }
