@@ -1,24 +1,24 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*-
 
    nemo-window-slot.c: Nemo window slot
- 
+
    Copyright (C) 2008 Free Software Foundation, Inc.
-  
+
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.
-  
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.
-  
+
    You should have received a copy of the GNU General Public
    License along with this program; if not, write to the
    Free Software Foundation, Inc., 51 Franklin Street - Suite 500,
    Boston, MA 02110-1335, USA.
-  
+
    Author: Christian Neumair <cneumair@gnome.org>
 */
 
@@ -39,6 +39,9 @@
 #include <libnemo-private/nemo-file-utilities.h>
 #include <libnemo-private/nemo-global-preferences.h>
 
+#define DEBUG_FLAG NEMO_DEBUG_WINDOW
+#include <libnemo-private/nemo-debug.h>
+
 #include <eel/eel-string.h>
 
 G_DEFINE_TYPE (NemoWindowSlot, nemo_window_slot, GTK_TYPE_BOX);
@@ -65,6 +68,13 @@ sync_search_directory (NemoWindowSlot *slot)
 	g_assert (NEMO_IS_SEARCH_DIRECTORY (directory));
 
 	query = nemo_query_editor_get_query (slot->query_editor);
+
+    if (query) {
+        nemo_query_set_show_hidden (query,
+                                    g_settings_get_boolean (nemo_preferences,
+                                                            NEMO_PREFERENCES_SHOW_HIDDEN_FILES));
+    }
+
 	nemo_search_directory_set_query (NEMO_SEARCH_DIRECTORY (directory),
 					     query);
 
@@ -235,6 +245,7 @@ real_active (NemoWindowSlot *slot)
     nemo_window_sync_bookmark_action (window);
 	nemo_window_pane_sync_location_widgets (slot->pane);
 	nemo_window_pane_sync_search_widgets (slot->pane);
+	nemo_window_sync_thumbnail_action(window);
 
 	if (slot->viewed_file != NULL) {
 		nemo_window_sync_view_type (window);
@@ -277,7 +288,7 @@ nemo_window_slot_init (NemoWindowSlot *slot)
 
 	slot->query_editor = NEMO_QUERY_EDITOR (nemo_query_editor_new ());
 
-	nemo_window_slot_add_extra_location_widget (slot, slot->query_editor);
+	nemo_window_slot_add_extra_location_widget (slot, GTK_WIDGET (slot->query_editor));
 
 	slot->view_overlay = gtk_overlay_new ();
 	gtk_widget_add_events (slot->view_overlay,
@@ -407,7 +418,7 @@ nemo_window_slot_class_init (NemoWindowSlotClass *klass)
 			      NULL, NULL,
 			      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE, 0);
-	
+
 	signals[CHANGED_PANE] =
 		g_signal_new ("changed-pane",
 			G_TYPE_FROM_CLASS (klass),
@@ -455,7 +466,7 @@ void
 nemo_window_slot_make_hosting_pane_active (NemoWindowSlot *slot)
 {
 	g_assert (NEMO_IS_WINDOW_PANE (slot->pane));
-	
+
 	nemo_window_set_active_slot (nemo_window_slot_get_window (slot),
 					 slot);
 }
@@ -468,11 +479,11 @@ nemo_window_slot_get_window (NemoWindowSlot *slot)
 }
 
 /* nemo_window_slot_update_title:
- * 
+ *
  * Re-calculate the slot title.
  * Called when the location or view has changed.
  * @slot: The NemoWindowSlot in question.
- * 
+ *
  */
 void
 nemo_window_slot_update_title (NemoWindowSlot *slot)
@@ -507,7 +518,7 @@ nemo_window_slot_update_title (NemoWindowSlot *slot)
 }
 
 /* nemo_window_slot_update_icon:
- * 
+ *
  * Re-calculate the slot icon
  * Called when the location or view or icon set has changed.
  * @slot: The NemoWindowSlot in question.
@@ -531,22 +542,32 @@ nemo_window_slot_update_icon (NemoWindowSlot *slot)
 			 * if we're setting to the same icon. This happens a lot e.g. when
 			 * the trash directory changes due to the file count changing.
 			 */
-			if (g_strcmp0 (icon_name, gtk_window_get_icon_name (GTK_WINDOW (window))) != 0) {			
+			if (g_strcmp0 (icon_name, gtk_window_get_icon_name (GTK_WINDOW (window))) != 0) {
 				gtk_window_set_icon_name (GTK_WINDOW (window), icon_name);
 			}
 		} else {
 			pixbuf = nemo_icon_info_get_pixbuf_nodefault (info);
-			
+
 			if (pixbuf) {
 				gtk_window_set_icon (GTK_WINDOW (window), pixbuf);
 				g_object_unref (pixbuf);
-			} 
+			}
 		}
-		
-		g_object_unref (info);
+
+        nemo_icon_info_unref (info);
 	}
 }
 
+void
+nemo_window_slot_set_show_thumbnails (NemoWindowSlot *slot,
+                                      gboolean show_thumbnails)
+{
+  NemoDirectory *directory;
+
+  directory = nemo_directory_get (slot->location);
+  nemo_directory_set_show_thumbnails(directory, show_thumbnails);
+}
+
 void
 nemo_window_slot_set_content_view_widget (NemoWindowSlot *slot,
 					      NemoView *new_view)
@@ -601,7 +622,7 @@ static void
 real_slot_set_short_status (NemoWindowSlot *slot,
 			    const gchar *status)
 {
-	
+
 	gboolean show_statusbar;
 	gboolean disable_chrome;
 
@@ -770,7 +791,7 @@ nemo_window_slot_get_current_view (NemoWindowSlot *slot)
 void
 nemo_window_slot_go_home (NemoWindowSlot *slot,
 			      NemoWindowOpenFlags flags)
-{			      
+{
 	GFile *home;
 
 	g_return_if_fail (NEMO_IS_WINDOW_SLOT (slot));
@@ -785,6 +806,7 @@ nemo_window_slot_go_up (NemoWindowSlot *slot,
 			    NemoWindowOpenFlags flags)
 {
 	GFile *parent;
+	char * uri;
 
 	if (slot->location == NULL) {
 		return;
@@ -792,7 +814,47 @@ nemo_window_slot_go_up (NemoWindowSlot *slot,
 
 	parent = g_file_get_parent (slot->location);
 	if (parent == NULL) {
-		return;
+		if (g_file_has_uri_scheme (slot->location, "smb")) {
+			uri = g_file_get_uri (slot->location);
+
+            DEBUG ("Starting samba URI for navigation: %s", uri);
+
+			if (g_strcmp0 ("smb:///", uri) == 0) {
+				parent = g_file_new_for_uri ("network:///");
+			}
+			else {
+                GString *gstr;
+                char * temp;
+
+                gstr = g_string_new (uri);
+
+				// Remove last /
+                if (g_str_has_suffix (gstr->str, "/")) {
+                    gstr = g_string_set_size (gstr, gstr->len - 1);
+                }
+
+				// Remove last part of string after last remaining /
+				temp = g_strrstr (gstr->str, "/") + 1;
+				if (temp != NULL) {
+                    gstr = g_string_set_size (gstr, temp - gstr->str);
+				}
+
+                // if we're going to end up with smb://, redirect it to network instead.
+                if (g_strcmp0 ("smb://", gstr->str) == 0) {
+                    gstr = g_string_assign (gstr, "network:///");
+                }
+
+                uri = g_string_free (gstr, FALSE);
+
+				parent = g_file_new_for_uri (uri);
+
+                DEBUG ("Ending samba URI for navigation: %s", uri);
+			}
+			g_free (uri);
+		}
+		else {
+			return;
+		}
 	}
 
 	nemo_window_slot_open_location (slot, parent, flags);
@@ -825,7 +887,7 @@ nemo_window_slot_should_close_with_mount (NemoWindowSlot *slot,
 	gboolean close_with_mount;
 
 	mount_location = g_mount_get_root (mount);
-	close_with_mount = 
+	close_with_mount =
 		g_file_has_prefix (NEMO_WINDOW_SLOT (slot)->location, mount_location) ||
 		g_file_equal (NEMO_WINDOW_SLOT (slot)->location, mount_location);
 
