@@ -72,12 +72,6 @@ enum {
     SORT_DESCENDING = 1
 };
 
-enum {
-    TIGHTER_NULL = -1,
-    TIGHTER_NO = 0,
-    TIGHTER_YES = 1
-};
-
 #define NEMO_WINDOW_SIDEBAR_PLACES "places"
 #define NEMO_WINDOW_SIDEBAR_TREE "tree"
 
@@ -119,6 +113,8 @@ void             nemo_window_disconnect_content_view (NemoWindow    *window,
 
 void             nemo_window_go_to                (NemoWindow    *window,
                                                        GFile             *location);
+void             nemo_window_go_to_tab            (NemoWindow    *window,
+                                                       GFile             *location);
 void             nemo_window_go_to_full           (NemoWindow    *window,
                                                        GFile             *location,
                                                        NemoWindowGoToCallback callback,
@@ -187,8 +183,6 @@ const gchar *nemo_window_get_ignore_meta_sort_column (NemoWindow *window);
 void         nemo_window_set_ignore_meta_sort_column (NemoWindow *window, const gchar *column);
 gint         nemo_window_get_ignore_meta_sort_direction (NemoWindow *window);
 void         nemo_window_set_ignore_meta_sort_direction (NemoWindow *window, gint direction);
-gint         nemo_window_get_ignore_meta_tighter_layout (NemoWindow *window);
-void         nemo_window_set_ignore_meta_tighter_layout (NemoWindow *window, gint tighter);
 
 NemoWindowOpenFlags nemo_event_get_window_open_flags   (void);
 
